package bodycheck

import (
	"encoding/json"
	"io/ioutil"
	"spaceofcheng/services/controllers"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/service/bodycheck"
	"spaceofcheng/services/utils"
	"strconv"
	"time"

	"github.com/astaxie/beego"

	"github.com/astaxie/beego/config"
	"github.com/zjxpcyc/wechat/wx"
)

// BodyCheckController 应用
type BodyCheckController struct {
	dao *bodycheck.BodycheckServ
	controllers.BaseController
	weixin *wx.Client
}

// Constructor 初始化 Controller
// @Title Constructor
// @Description 初始化 Controller, 系统自动调用
func (c *BodyCheckController) Constructor() {
	c.dao = bodycheck.NewBodycheckServ(c.Context)
	org := c.Context.Get("org").(model.SysOrg)
	c.weixin = utils.WxClientFor(org.OrgId)
}

// GetCheckByUser 根据用户获取体检报告
func (c *BodyCheckController) GetCheckByUser() {
	code := c.GetString("code")
	openid, err := c.weixin.GetOpenID(code)
	if err != nil {
		beego.Error("获取openid失败！", err.Error())
		c.ResponseError(err)
	}
	// openid := "o5EGQ0ytw9JUpSa6N84ddo3CYFO4"
	user, err := c.dao.GetUserByOpenID(openid)
	if err != nil {
		c.ResponseError(err)
	}
	if user == nil {
		c.ResponseError(err)
	}

	bodyCheck, err := c.dao.GetBodyCheckByUser(user.CustomerId)
	if err != nil {
		c.ResponseJson(map[string]interface{}{
			"Status":  false,
			"Message": err.Error(),
		})
	}
	if len(bodyCheck) == 0 {
		c.ResponseJson(map[string]interface{}{
			"Status":  false,
			"Message": "没有查询到数据",
		})
	}
	presentations, err := c.dao.GetPresentationByCheckID(bodyCheck[0].Id)
	if err != nil {
		c.ResponseJson(map[string]interface{}{
			"Status":  false,
			"Message": err.Error(),
		})
	}
	c.ResponseJson(map[string]interface{}{
		"Status":   true,
		"Message":  presentations,
		"Info":     bodyCheck,
		"UserInfo": user,
	})
}

// PostCheckResult 测量结果返回
func (c *BodyCheckController) PostCheckResult() {
	r := c.Ctx.Request
	defer r.Body.Close()
	con, _ := ioutil.ReadAll(r.Body) //获取post的数据

	var formVal map[string]interface{}
	err := json.Unmarshal(con, &formVal)
	if err != nil {
		beego.Error(err)
		c.ResponseJson(map[string]interface{}{
			"Status":  false,
			"Message": err.Error(),
		})
	}

	type CheckResult struct {
		WechatAccount string
		EquipmentId   string
		CheckType     int
		CheckDate     time.Time
		CheckResult   string
		ReportUrl     string
		HMSReportUrl  string
	}

	var result = CheckResult{}

	result.EquipmentId = formVal["EquipmentId"].(string)
	result.WechatAccount = formVal["WechatAccount"].(string)
	// result.CheckResult = formVal["CheckResult"].(string)
	result.CheckResult = ""
	result.ReportUrl = formVal["ReportUrl"].(string)
	result.HMSReportUrl = formVal["HMSReportUrl"].(string)
	result.CheckType = int(formVal["CheckType"].(float64))
	if err != nil {
		beego.Error(err.Error())
		c.ResponseJson(map[string]interface{}{
			"Status":  false,
			"Message": err.Error(),
		})
	}
	loc, _ := time.LoadLocation("Local")
	result.CheckDate, err = time.ParseInLocation("2006-01-02 15:04:05", formVal["CheckDate"].(string), loc)
	if err != nil {
		beego.Error(err.Error())
		c.ResponseJson(map[string]interface{}{
			"Status":  false,
			"Message": err.Error(),
		})
	}

	user, err := c.dao.GetUserByOpenID(result.WechatAccount)
	if err != nil {
		beego.Error(err.Error())
		c.ResponseJson(map[string]interface{}{
			"Status":  false,
			"Message": err.Error(),
		})
	}

	if user == nil || user.Id < 1 {
		beego.Error("没有当前用户信息！")
		c.ResponseJson(map[string]interface{}{
			"Status":  false,
			"Message": "没有当前用户信息！",
		})
	}
	userid := user.Id

	checkinfo, err := c.dao.GetCheckByUserAndEquipmentID(userid, result.EquipmentId)
	if err != nil {
		beego.Error(err.Error())
		c.ResponseJson(map[string]interface{}{
			"Status":  false,
			"Message": err.Error(),
		})
	}
	if checkinfo == nil || checkinfo.Id < 1 || time.Now().Local().Format("2006-01-02") != checkinfo.CreateDate.Format("2006-01-02") {
		caseEquipment, err := c.dao.GetCaseEquipment(result.EquipmentId)
		if err != nil {
			beego.Error(err.Error())
			c.ResponseJson(map[string]interface{}{
				"Status":  false,
				"Message": err.Error(),
			})
		}
		if caseEquipment == nil || caseEquipment.Id < 1 {
			beego.Error("设备未维护！")
			c.ResponseJson(map[string]interface{}{
				"Status":  false,
				"Message": "设备未维护！",
			})
		}
		var checkNew = model.TaBodyCheck{}
		checkNew.CaseId = caseEquipment.CaseId
		checkNew.EquipmentId = result.EquipmentId
		checkNew.UserId = userid
		checkNew.ReportUrl = result.HMSReportUrl
		checkNew.CreateDate = result.CheckDate
		checkinfo, err = c.dao.SaveBodyCheckInfo(checkNew)

		wxconf, _ := config.NewConfig("ini", "conf/wechat.conf")
		messageTplID := wxconf.String("messageTplID")

		c.weixin.SendTplMessage(result.WechatAccount, messageTplID, beego.AppConfig.String("resultURL"), map[string]wx.TplMessageData{
			"first": wx.TplMessageData{
				Value: "您的体检报告已生成",
			},
			"keyword1": wx.TplMessageData{
				Value: strconv.Itoa(checkinfo.Id),
			},
			"keyword2": wx.TplMessageData{
				Value: result.CheckDate.Format("2006-01-02 15:04"),
			},
			"remark": wx.TplMessageData{
				Value: "",
			},
		})
	}

	presentation, err := c.dao.GetPresentation(checkinfo.Id, result.CheckType)
	if err != nil {
		beego.Error(err.Error())
		c.ResponseJson(map[string]interface{}{
			"Status":  false,
			"Message": err.Error(),
		})
	}
	if presentation == nil || presentation.Id < 1 {
		// 新增
		var preNew = model.TaPresentation{}
		preNew.CheckId = checkinfo.Id
		preNew.CheckDate = result.CheckDate
		preNew.CheckResult = result.CheckResult
		preNew.CheckType = result.CheckType
		preNew.ReportUrl = result.ReportUrl
		presentation, err = c.dao.SavePresentation(preNew)
		if err != nil {
			beego.Error(err.Error())
			c.ResponseJson(map[string]interface{}{
				"Status":  false,
				"Message": err.Error(),
			})
		}
	} else {
		// 修改
		presentation.CheckDate = result.CheckDate
		presentation.CheckResult = result.CheckResult
		presentation.ReportUrl = result.ReportUrl
		err = c.dao.UpdatePresentation(presentation)
		if err != nil {
			beego.Error(err.Error())
			c.ResponseJson(map[string]interface{}{
				"Status":  false,
				"Message": err.Error(),
			})
		}
		// 删除之前的记录
		err = c.dao.DeletePresentionDetail(presentation.Id)
		if err != nil {
			beego.Error(err.Error())
			c.ResponseJson(map[string]interface{}{
				"Status":  false,
				"Message": err.Error(),
			})
		}
	}

	specs, err := c.dao.GetCheckSpecs()
	if err != nil {
		beego.Error(err.Error())
		c.ResponseJson(map[string]interface{}{
			"Status":  false,
			"Message": err.Error(),
		})
	}

	if len(formVal) > 0 {
		var details []model.TaPresentationDetail
		for k, v := range formVal {
			if k != "WechatAccount" && k != "EquipmentId" && k != "CheckType" && k != "CheckDate" && k != "CheckResult" && k != "ReportUrl" && k != "HMSReportUrl" && k != "WXID" {
				var detail model.TaPresentationDetail
				detail.CheckName = k
				switch vx := v.(type) {
				case string:
					detail.CheckVal = vx
				case float64:
					detail.CheckVal = strconv.FormatFloat(vx, 'f', 2, 64)
				}

				detail.PresentationId = presentation.Id
				var specName = ""
				for _, spec := range specs {
					if spec.SortName == k {
						specName = spec.Name
						break
					}
				}
				detail.SpecName = specName
				details = append(details, detail)
			}
		}
		if len(details) > 0 {
			err := c.dao.SavePresentationDetail(details)
			if err != nil {
				beego.Error(err.Error())
				c.ResponseJson(map[string]interface{}{
					"Status":  false,
					"Message": err.Error(),
				})
			}
		}
	}
	c.ResponseJson(map[string]interface{}{
		"Status":  true,
		"Message": "",
	})
}
