package luckdraw

import (
	"spaceofcheng/services/controllers"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/service/luckdraw"
	"strconv"
)

// LuckdrawController 应用
type LuckdrawController struct {
	dao *luckdraw.LuckdrawServ
	controllers.BaseController
}

// Constructor 初始化 Controller
// @Title Constructor
// @Description 初始化 Controller, 系统自动调用
func (c *LuckdrawController) Constructor() {
	c.dao = luckdraw.NewLuckdrawServ(c.Context)
	org := c.Context.Get("org").(model.SysOrg)
}

// GetUserByCode 根据code获取用户信息
func (c *LuckDrawController) GetUserByCode() {
	custID := 0
	encodeUser, _ := c.GetInt("from")
	if encodeUser != 0 {
		encodeUser = helper.IntXOR(encodeUser, 97)
		enStr := strconv.Itoa(encodeUser)
		enStr = enStr[:len(enStr)-3]

		custID, _ = strconv.Atoi(enStr)
	}

	code := c.GetString("code")
	if c.openid == "" && code == "" && custID == 0 {
		c.ResponseJson(map[string]interface{}{
			"Status":  false,
			"Message": "401-" + c.weixin.GetAppID(),
		})
	}

	var user *model.CustomerUser
	if custID != 0 {
		var err error
		user, err = c.bodydao.GetCustomerByID(custID)
		if err != nil {
			c.ResponseJson(map[string]interface{}{
				"Status":  false,
				"Message": helper.LogError("获取用户信息失败", err.Error()).Error(),
			})
		}

		c.openid = user.OpenId
	}

	if c.openid == "" {
		openid, err := c.weixin.GetOpenID(code)
		if err != nil {
			c.ResponseJson(map[string]interface{}{
				"Status":  false,
				"Message": helper.LogError("获取微信 OpenId 失败", err.Error()).Error(),
			})
		}

		c.openid = openid
		c.SetSession("openid", openid)
	}

	if user == nil {
		var err error
		user, err = c.bodydao.GetUserByOpenID(c.openid)
		if err != nil {
			c.ResponseJson(map[string]interface{}{
				"Status":  false,
				"Message": helper.LogError("获取用户信息失败", err.Error()).Error(),
			})
		}
	}

	if user == nil || user.Tel == "" {
		c.ResponseJson(map[string]interface{}{
			"Status":  true,
			"Message": 406,
		})
	}

	luckdrawid, _ := c.GetInt("luckdrawid")
	record, err := c.dao.GetUserLuckDrawByLuckDraw(user.Id, luckdrawid)
	if err != nil {
		c.ResponseJson(map[string]interface{}{
			"Status":  false,
			"Message": helper.LogError("获取用户抽奖信息失败", err.Error()).Error(),
		})
	}
	var detail *model.TaPrizeDetail
	if record != nil {
		detail, err = c.dao.GetDetailByRecord(record.Id)
		if err != nil {
			c.ResponseJson(map[string]interface{}{
				"Status":  false,
				"Message": helper.LogError("获取用户抽奖信息失败", err.Error()).Error(),
			})
		}
	}

	c.ResponseJson(map[string]interface{}{
		"Status": true,
		"Message": map[string]interface{}{
			"user":   user,
			"record": record,
			"detail": detail,
		},
	})
}

// GetLuckDraw 获取抽奖信息
func (c *LuckDrawController) GetLuckDraw() {
	id, _ := c.GetInt(":id")
	luckdraw, err := c.dao.GetLuckDraw(id)
	if err != nil {
		c.ResponseJson(map[string]interface{}{
			"Status":  false,
			"Message": helper.LogError("获取抽奖信息失败", err.Error()).Error(),
		})
	}
	c.ResponseJson(map[string]interface{}{
		"Status":  true,
		"Message": luckdraw,
	})
}

// LuckDraw 抽奖
func (c *LuckDrawController) LuckDraw() {
	if c.openid == "" {
		c.ResponseJson(map[string]interface{}{
			"Status":  false,
			"Message": "401-" + c.weixin.GetAppID(),
		})
	}

	id, _ := c.GetInt(":id")
	user, err := c.bodydao.GetUserByOpenID(c.openid)
	if err != nil {
		c.ResponseJson(map[string]interface{}{
			"Status":  false,
			"Message": helper.LogError("获取用户信息失败", err.Error()).Error(),
		})
	}

	prize, detail, err := c.dao.LuckDraw(id, user)
	if err != nil {
		c.ResponseJson(map[string]interface{}{
			"Status":  false,
			"Message": err.Error(),
		})
	}
	c.ResponseJson(map[string]interface{}{
		"Status": true,
		"Message": map[string]interface{}{
			"prize":  prize,
			"detail": detail,
		},
	})
}

// GetRecordByLuckDraw 获取抽奖记录
func (c *LuckDrawController) GetRecordByLuckDraw() {
	id, _ := c.GetInt("id")
	records, err := c.dao.GetRecordByLuckDraw(id)
	if err != nil {
		c.ResponseJson(map[string]interface{}{
			"Status":  false,
			"Message": helper.LogError("获取抽奖记录失败", err.Error()).Error(),
		})
	}
	c.ResponseJson(map[string]interface{}{
		"Status":  true,
		"Message": records,
	})
}

// GetUserLuckDraw 获取用户的抽奖记录
func (c *LuckDrawController) GetUserLuckDraw() {
	userid, _ := c.GetInt("userid")
	records, err := c.dao.GetUserLuckDraw(userid)
	if err != nil {
		c.ResponseJson(map[string]interface{}{
			"Status":  false,
			"Message": helper.LogError("获取用户抽奖记录失败", err.Error()).Error(),
		})
	}
	c.ResponseJson(map[string]interface{}{
		"Status":  true,
		"Message": records,
	})
}

// GetRecordByID 根据ID获取中奖记录信息
func (c *LuckDrawController) GetRecordByID() {
	id, _ := c.GetInt(":id")
	record, err := c.dao.GetRecordByID(id)
	if err != nil {
		c.ResponseJson(map[string]interface{}{
			"Status":  false,
			"Message": helper.LogError("获取中奖记录失败", err.Error()).Error(),
		})
	}

	caseAddress, err := c.dao.GetCaseAddressByLuckDarw(record.LuckdrawId)
	if err != nil {
		c.ResponseJson(map[string]interface{}{
			"Status":  false,
			"Message": helper.LogError("获取案场信息失败", err.Error()).Error(),
		})
	}

	if caseAddress == "" {
		c.ResponseJson(map[string]interface{}{
			"Status":  false,
			"Message": "无案场, 或案场地址没有维护",
		})
	}

	prize, err := c.dao.GetPrizeByID(record.PrizeId)
	if err != nil {
		c.ResponseJson(map[string]interface{}{
			"Status":  false,
			"Message": helper.LogError("校验奖品记录失败", err.Error()).Error(),
		})
	}

	c.ResponseJson(map[string]interface{}{
		"Status": true,
		"Message": map[string]interface{}{
			"record":      record,
			"prize":       prize,
			"caseAddress": caseAddress,
		},
	})
}

func (c *LuckDrawController) SaveShareRecord() {
	if c.openid == "" {
		c.ResponseJson(map[string]interface{}{
			"Status":  false,
			"Message": "401-" + c.weixin.GetAppID(),
		})
	}

	errPrefix := "logerror-"

	id, _ := c.GetInt(":id")

	fromID, _ := c.GetInt("from")
	if fromID == 0 {
		c.ResponseJson(map[string]interface{}{
			"Status":  false,
			"Message": errPrefix + "没有分享人ID",
		})
	}

	from, err := c.bodydao.GetCustomerByID(fromID)
	if err != nil {
		c.ResponseJson(map[string]interface{}{
			"Status":  false,
			"Message": errPrefix + helper.LogError("获取分享人信息失败", err.Error()).Error(),
		})
	}

	if from == nil || from.Id == 0 {
		c.ResponseJson(map[string]interface{}{
			"Status":  false,
			"Message": errPrefix + helper.LogError("分享人信息不存在").Error(),
		})
	}

	current, err := c.bodydao.GetUserByOpenID(c.openid)
	if err != nil {
		c.ResponseJson(map[string]interface{}{
			"Status":  false,
			"Message": errPrefix + helper.LogError("获取当前人员信息失败", err.Error()).Error(),
		})
	}

	if current == nil || current.Id == 0 {
		c.ResponseJson(map[string]interface{}{
			"Status":  false,
			"Message": errPrefix + "系统错误, 无当前人员信息",
		})
	}

	// 自己分享给自己
	if from.Id == current.Id {
		c.ResponseJson(map[string]interface{}{
			"Status":  false,
			"Message": errPrefix + "本人分享内容无效",
		})
	}

	luckdraw, err := c.dao.GetLuckDrawByID(id)
	if err != nil {
		c.ResponseJson(map[string]interface{}{
			"Status":  false,
			"Message": errPrefix + helper.LogError("获取抽奖信息失败", err.Error()).Error(),
		})
	}

	if luckdraw == nil {
		c.ResponseJson(map[string]interface{}{
			"Status":  false,
			"Message": errPrefix + helper.LogError("没有找到 id ("+strconv.Itoa(id)+") 对应的抽奖信息").Error(),
		})
	}

	existsRec, err := c.dao.GetLuckDrawShareData(from.Id, current.Id, luckdraw.Id, luckdraw.CaseId)
	if err != nil {
		c.ResponseJson(map[string]interface{}{
			"Status":  false,
			"Message": errPrefix + helper.LogError("获取已有分享信息失败", err.Error()).Error(),
		})
	}

	if existsRec != nil && len(existsRec) > 0 {
		c.ResponseJson(map[string]interface{}{
			"Status":  false,
			"Message": errPrefix + "已分享记录, 再次分享无效",
		})
	}

	shareDt := model.TaShareLuckyRecord{
		FromCustomerId:     from.Id,
		FromCustomerName:   from.Username,
		FromCustomerWxname: from.WcNickname,
		FromCustomerTel:    from.Tel,
		ToCustomerId:       current.Id,
		ToCustomerName:     current.Username,
		ToCustomerWxname:   current.WcNickname,
		ToCustomerTel:      current.Tel,
		CaseId:             luckdraw.CaseId,
		LuckydrawId:        luckdraw.Id,
		LuckydrawName:      luckdraw.Name,
	}

	if err := c.dao.SaveLuckDrawShareData(&shareDt); err != nil {
		c.ResponseJson(map[string]interface{}{
			"Status":  false,
			"Message": errPrefix + helper.LogError("保存分享记录失败", err.Error()).Error(),
		})
	}

	c.ResponseJson(map[string]interface{}{
		"Status":  true,
		"Message": "ok",
	})
}
